unit FormTemplate;

interface

uses
  Contnrs,
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, IniFiles;

type
  //-   
  TfmTemplate = class (TForm)
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
  protected
    IniFile: TIniFile;
    ChildList: TObjectList;
    procedure Init; virtual;
    procedure PrepareForm; virtual;
    procedure WriteIniFile(AOwner: TComponent; SectionName: string = ''); virtual;
    procedure ReadIniFile(AOwner: TComponent; SectionName: string = ''); virtual;
  public
    procedure AfterConstruction; override;
    procedure BeforeDestruction; override;
  end;

implementation

uses StdCtrls, ExtCtrls, DbGrids, DBGridEh, Spin, FormWait;

{$R *.dfm}


//-   
procedure TfmTemplate.AfterConstruction;
begin
  ChildList := TObjectList.Create(True);
  Init;
  inherited;
end;

procedure TfmTemplate.BeforeDestruction;
begin
  inherited;
  if Assigned(IniFile) then IniFile.Free;
  if Assigned(ChildList) then ChildList.Free;
end;

procedure TfmTemplate.FormShow(Sender: TObject);
begin
  PrepareForm;
end;

procedure TfmTemplate.Init;
begin

end;

procedure TfmTemplate.PrepareForm;
begin
  ReadIniFile(Self);
end;

procedure TfmTemplate.ReadIniFile(AOwner: TComponent; SectionName: string = '');
var i,j,k: integer;
    c: TComponent;
begin
  if not Assigned(IniFile) then IniFile := TIniFile.Create(ChangeFileExt(ParamStr(0), '.ini'));
  if SectionName = '' then SectionName := AOwner.ClassName;
  with AOwner do
  begin

    if Tag = -1 then
    begin
      Left := IniFile.ReadInteger(SectionName, 'Left', Left);
      Top := IniFile.ReadInteger(SectionName, 'Top', Top);
      Height := IniFile.ReadInteger(SectionName, 'Height', Height);
      Width := IniFile.ReadInteger(SectionName, 'Width', Width);
      WindowState := TWindowState(IniFile.ReadInteger(SectionName, 'WindowState', Integer(wsNormal)));
    end;
    for i := 0 to ComponentCount - 1 do
    begin
      c := Components[i];

      if c.Tag = -1 then //       
      begin

//        if c is TfrmTemplate then
//        begin
//          ReadIniFile(c, SectionName + '.' + c.ClassName);
//          TfrmTemplate(c).ReadIniFile(IniFile, c, SectionName + '.' + c.ClassName);
//        end;

        if c is TEdit then
        with c as TEdit do
        begin
          TEdit(c).Text := IniFile.ReadString(SectionName, c.Name, TEdit(c).Text);
        end;

        if c is TSpinEdit then
        with c as TSpinEdit do
        begin
          TSpinEdit(c).Value := IniFile.ReadInteger(SectionName, c.Name, TSpinEdit(c).Value);
        end;

        if c is TGroupBox then
        with c as TGroupBox do
        begin
          if Align in [alRight,alLeft] then Width := IniFile.ReadInteger(SectionName, c.Name, TGroupBox(c).Width);
          if Align in [alTop,alBottom] then Height := IniFile.ReadInteger(SectionName, c.Name, TGroupBox(c).Height);
        end;

        if c is TPanel then
        with c as TPanel do
        begin
          if Align in [alRight,alLeft] then Width := IniFile.ReadInteger(SectionName, c.Name, TPanel(c).Width);
          if Align in [alTop,alBottom] then Height := IniFile.ReadInteger(SectionName, c.Name, TPanel(c).Height);
        end;

        if c is TDBGrid then
        with c as TDBGrid do
        begin
          if Align in [alRight,alLeft] then Width := IniFile.ReadInteger(SectionName, c.Name, TDBGrid(c).Width);
          if Align in [alTop,alBottom] then Height := IniFile.ReadInteger(SectionName, c.Name, TDBGrid(c).Height);
          for j := 0 to Columns.Count - 1 do
            Columns[j].Width := IniFile.ReadInteger(SectionName, c.Name + '.Column' + IntToStr(j), Columns[j].Width);
        end;

        if c is TDBGridEh then
        with c as TDBGridEh do
        begin
          if TDBGridEh(c).Align in [alRight,alLeft] then TDBGridEh(c).Width := IniFile.ReadInteger(SectionName, c.Name, TDBGridEh(c).Width);
          if TDBGridEh(c).Align in [alTop,alBottom] then TDBGridEh(c).Height := IniFile.ReadInteger(SectionName, c.Name, TDBGridEh(c).Height);
          for j := 0 to Columns.Count - 1 do
            Columns[j].Width := IniFile.ReadInteger(SectionName, c.Name + '.Column' + IntToStr(j), Columns[j].Width);
        end;
      end;
    end;
  end;
end;

procedure TfmTemplate.WriteIniFile(AOwner: TComponent; SectionName: string = '');
var i,j,k: integer;
    c: TComponent;
begin
  Wait(True, ' ...');
  if not Assigned(IniFile) then IniFile := TIniFile.Create(ChangeFileExt(ParamStr(0), '.ini'));
  try
    if SectionName = '' then SectionName := AOwner.ClassName;
    with AOwner do
    begin
      if Tag = -1 then
      begin
        IniFile.WriteInteger(SectionName, 'Left', Left);
        IniFile.WriteInteger(SectionName, 'Top', Top);
        IniFile.WriteInteger(SectionName, 'Height', Height);
        IniFile.WriteInteger(SectionName, 'Width', Width);
        IniFile.WriteInteger(SectionName, 'WindowState', Integer(WindowState));
      end;
      for i := 0 to ComponentCount - 1 do
      begin
        c := Components[i];
        if c.Tag = -1 then //       
        begin
//          if c is TfrmTemplate then
//          begin
//            WriteIniFile(c, SectionName + '.' + c.ClassName);
//            TfrmTemplate(c).WriteIniFile(IniFile, c, SectionName + '.' + c.ClassName);
//          end;

          if c is TEdit then
          with c as TEdit do
          begin
            IniFile.WriteString(SectionName, c.Name, TEdit(c).Text);
          end;

          if c is TSpinEdit then
          with c as TSpinEdit do
          begin
            IniFile.WriteInteger(SectionName, c.Name, TSpinEdit(c).Value);
          end;

          if c is TGroupBox then
          with c as TGroupBox do
          begin
            if Align in [alRight,alLeft] then IniFile.WriteInteger(SectionName, c.Name, TGroupBox(c).Width);
            if Align in [alTop,alBottom] then IniFile.WriteInteger(SectionName, c.Name, TGroupBox(c).Height);
          end;

          if c is TPanel then
          with c as TPanel do
          begin
            if Align in [alRight,alLeft] then IniFile.WriteInteger(SectionName, c.Name, TPanel(c).Width);
            if Align in [alTop,alBottom] then IniFile.WriteInteger(SectionName, c.Name, TPanel(c).Height);
          end;

          if c is TDBGrid then
          with c as TDBGrid do
          begin
            if Align in [alRight,alLeft] then IniFile.WriteInteger(SectionName, c.Name, TDBGrid(c).Width);
            if Align in [alTop,alBottom] then IniFile.WriteInteger(SectionName, c.Name, TDBGrid(c).Height);
            for j := 0 to Columns.Count - 1 do
              IniFile.WriteInteger(Owner.Name, c.Name + '.Column' + IntToStr(j), Columns[j].Width);
          end;

          if c is TDBGridEh then
          with c as TDBGridEh do
          begin
            if Align in [alRight,alLeft] then IniFile.WriteInteger(SectionName, c.Name, TDBGridEh(c).Width);
            if Align in [alTop,alBottom] then IniFile.WriteInteger(SectionName, c.Name, TDBGridEh(c).Height);
            for j := 0 to Columns.Count - 1 do
              IniFile.WriteInteger(SectionName, c.Name + '.Column' + IntToStr(j), Columns[j].Width);
          end;
        end;
      end;
    end;
  finally
    Wait(False);
  end;
end;

procedure TfmTemplate.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  WriteIniFile(Self);
end;

end.


