unit FormWait;

interface

uses
  FormTemplate,                               
  Classes, Forms,
  StdCtrls, PropStorageEh, Controls, ExtCtrls, ComCtrls, Buttons;

type
  TWaitProc = procedure of object;
  TDetailProc = procedure (Details: TStrings) of object;

  TfmWait = class(TfmTemplate)
    lblText: TLabel;
    ProgressBar: TProgressBar;
    Panel: TPanel;
    Memo: TMemo;
    btnDetail: TButton;
    btnBreak: TBitBtn;
    lblTime: TLabel;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnBreakClick(Sender: TObject);
    procedure btnDetailClick(Sender: TObject);
  private
    start_tick: Integer;
    FBreakProc: TWaitProc;
    FDetailProc: TDetailProc;
    procedure SetBreakProc(const Value: TWaitProc);
    procedure SetDetailProc(const Value: TDetailProc);
    { Private declarations }

  public
    { Public declarations }
    procedure ProgressInit(min, max: integer);
    procedure ProgressDoStep;
    property BreakProc: TWaitProc read FBreakProc write SetBreakProc;
    property DetailProc: TDetailProc read FDetailProc write SetDetailProc;
  end;

var
  fmWait: TfmWait;

function Wait(isShow: boolean = True; Msg: string = ' ...'): TfmWait;

implementation
{$R *.dfm}

uses Windows, SysUtils;


function Wait(isShow: boolean = True; Msg: string = ' ...'): TfmWait;
begin
  if (isShow) then
  begin
    Screen.Cursor := crHourGlass;
    if not Assigned(fmWait) then fmWait := TfmWait.Create(Application);
    fmWait.AutoSize := False;
    fmWait.lblText.AutoSize := False;
    fmWait.Width := fmWait.lblText.Canvas.TextWidth(Msg) + 30;
    fmWait.lblText.Caption := Msg;
    fmWait.lblText.AutoSize := True;
    fmWait.AutoSize := True;
    fmWait.Show;
    fmWait.BringToFront;
    fmWait.Position := poScreenCenter;
    fmWait.Memo.Lines.Append(Msg);
    Result := fmWait;
    Application.ProcessMessages;
  end
  else
  begin
    Screen.Cursor := crDefault;
    if Assigned(fmWait) then
    begin
      fmWait.Free;
      fmWait := nil;
      Result := nil;
    end;
  end;
end;


{ TfmWait }

procedure TfmWait.ProgressDoStep;
var cur_tick: Integer;
    sec_between: integer;
    m,s: Integer;
begin
//  Application.ProcessMessages;
  cur_tick := GetTickCount;
  sec_between := (cur_tick - start_tick) div 1000;
  if ProgressBar.Position > 0 then sec_between := Round(ProgressBar.Max * sec_between / ProgressBar.Position) - sec_between;
  m := sec_between div 60;
  s := sec_between - m * 60;
  lblTime.Caption := ' ... ' + IntToStr(m) + ' .  ' + IntToStr(s) + ' .';
  ProgressBar.StepBy(1);
  Application.ProcessMessages;
end;

procedure TfmWait.ProgressInit(min, max: integer);
begin
  AutoSize := False;
  ProgressBar.Enabled := False;
  ProgressBar.Min := min;
  ProgressBar.Position := min;
  ProgressBar.Max := max;
  ProgressBar.Enabled := True;
  ProgressBar.Show;
  lblTime.Show;
  AutoSize := True;
  start_tick := GetTickCount;
  Application.ProcessMessages;
end;

procedure TfmWait.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  inherited;
  ProgressBar.Hide;
end;

procedure TfmWait.btnBreakClick(Sender: TObject);
begin
  inherited;
  if Assigned(BreakProc) then BreakProc;
end;

procedure TfmWait.SetBreakProc(const Value: TWaitProc);
begin
  FBreakProc := Value;
end;

procedure TfmWait.btnDetailClick(Sender: TObject);
begin
  inherited;
  AutoSize := False;
  Memo.Visible := not Memo.Visible;
  AutoSize := True;
  if Memo.Visible then btnDetail.Caption := ' <<'
  else btnDetail.Caption := ' >>';
  Application.ProcessMessages;
end;

procedure TfmWait.SetDetailProc(const Value: TDetailProc);
begin
  FDetailProc := Value;
end;

end.
