{{
 :  ..
 :    OLE  - WordBasic.
}
unit kaWordBasicWrapper;

interface

uses
 // C 
 Classes, SysUtils;

const
 iTrue  = 1;
 iFalse = 0;

 //    OLE :
 dmNone   = -1; // dmNone    -    ,      OLE 
 dmPrompt  = 0; // dmPrompt  -  Word          
 dmSaveAll = 1; // dmSaveAll -  Word  ,    
 dmQuit    = 2; // dmQuit    -  Word  ,     
type
 TkaWordDisconnectMode    = dmNone..dmQuit;
 TkaWordCloseDocumentMode = dmPrompt..dmQuit;

 //    TkaWordBasicWrapper
 EWordBasicException = class(Exception);

 //  Word-:
 //  wvOld -   '8.0'
 //  wvNew -    '8.0'
 TkaWordVersion = (wvOld, wvNew);

 //      OLE  Word.Basic
 TkaWordBasicWrapper = class(TObject)
 private
  WordBasic: variant;
  WordVersion: TkaWordVersion;

  function  GetWordVersionStr: string;      //   Word-   
  function  GetWordVersion: TkaWordVersion; //   Word-

  procedure CheckIsConnected;
 public
  SelStart, SelEnd: integer; //    

  constructor Create;
  destructor  Destroy; override;

  procedure Connect; //   OLE  Word.Basic
  procedure Disconnect(DisconnectMode: TkaWordDisconnectMode = dmPrompt); //   OLE  Word.Basic

  procedure CreateNewDocument(const TemplateName: string = '');
  procedure CloseDocument(Mode: TkaWordCloseDocumentMode = dmQuit); //   

  procedure Show(IsStartOfDoc: boolean = true); //  Word       
  procedure Hide; //  () Word

  procedure Undo; //   ,   
  procedure Redo; //   ,   

  procedure Select(psFrom, psTo: integer); //    psFrom  psTo
  procedure SelectCurrent;                 //    SelStart  SelEnd

  procedure FormatFont(FontSize, IsUnderline, IsBold, IsItalic: integer; FontName: string; ColorCode: integer = 0);
  procedure FormatAsBold(IsBold: boolean = true);
  procedure FormatAsUnderline(IsUnderline: boolean = true);

  procedure InsertText(const Text: string); //       
                                            //      SelStart, SelEnd

  procedure CopySelection;
  procedure CutSelection;
  procedure ClearSelection;
  procedure PasteFromClipboard;
  procedure MoveTo(const Dest: string; IsClear: boolean = true);
  procedure FindText(const TextToFind: string);
  procedure ReplaceText(const TextToFind, ReplaceWith: string; IsReplaceAll: boolean = true); overload;
  procedure ReplaceText(const TextToFind: string; ReplaceWith: integer; IsReplaceAll: boolean = true); overload;

  procedure TextToTable(TableBegin, TableEnd, TableColCount: integer);

  procedure TableSelectCell;
  procedure TableSelectRow;
  procedure TableSelectColumn;
  procedure TableSelectTable;

  procedure TableMergeCells;
  procedure TableSplitCells(Cols: integer = 2);

  procedure TableInsertRow(Rows: integer; IsGotoFirstCell: boolean = true);
  procedure TableInsertColumn(IsGotoFirstCell: boolean = true);
  procedure TableInsertText(const Text: string; IsNextCell: boolean = true); overload;
  procedure TableInsertText(const StringArray: array of string; IsNextCell: boolean = true); overload;

  function  StartOfRow(IsSelect: boolean = false): integer;
  function  EndOfRow(IsSelect: boolean = false): integer;
  function  StartOfColumn(IsSelect: boolean = false): integer;
  function  EndOfColumn(IsSelect: boolean = false): integer;
  procedure NextCell;
  procedure PrevCell;
  function  LineDown(Lines: integer = 1; IsSelect: boolean = false): integer;
  function  LineUp(Lines: integer = 1; IsSelect: boolean = false): integer;

  procedure AlignPara(Alignment: integer);

  procedure RemoveBullets;
  procedure SetBullets;

  procedure DelWord; //   (  )     ,     
                     //      ,       
                     //  Clipboard   

  procedure UpdateSelStart; //   SelStart
  procedure UpdateSelEnd;   //   SelEnd

  procedure StartOfDoc; //     
  procedure EndOfDoc;   //     

  procedure PrintDocument; //  

  //    :
  function CreateException(const Msg: string): Exception; overload;
  function CreateException(const Msg: string; E: Exception): Exception; overload;
  function CreateException(E: Exception; const Msg: string): Exception; overload;
  function CreateExceptionFmt(const Msg: string; const Args: array of const): Exception; overload;
  function CreateExceptionFmt(const Msg: string; const Args: array of const; E: Exception): Exception; overload;
  function CreateExceptionFmt(E: Exception; const Msg: string; const Args: array of const): Exception; overload;
 end;

const
 CRLF = #$D#$A;

implementation

uses
 // C 
 ComObj, Variants;

const
 S_OLE_OBJECT_CLASS_NAME = 'Word.Basic';

 S_POSSIBLY_NO_MS_WORD = ' Microsoft Word   ,         ';

 E_FAILED_TO_CONNECT_TO_MS_WORD         = '   ()  Microsoft Word';
 E_FAILED_TO_DISCONNECT_FROM_MS_WORD    = '   ()  Microsoft Word';
 E_FAILED_TO_GET_MS_WORD_VERSION        = '     Microsoft Word';
 E_NO_CONNECTION_ESTABLISHED            = '   Microsoft Word';
 E_FAILED_TO_CREATE_NEW_DOCUMENT        = '      : "%s"';
 E_FAILED_TO_CLOSE_DOCUMENT             = '   ';
 E_FAILED_TO_SHOW_MS_WORD               = '   Microsoft Word';
 E_FAILED_TO_HIDE_MS_WORD               = '   Microsoft Word';
 E_FAILED_TO_UNDO_LAST_ACTION           = '    ';
 E_FAILED_TO_REDO_LAST_ACTION           = '     ';
 E_FAILED_TO_INSERT_TEXT                = '   ';
 E_FAILED_TO_COPY_SELECTION             = '      ';
 E_FAILED_TO_CUT_SELECTION              = '      ';
 E_FAILED_TO_CLEAR_SELECTION            = '   ';
 E_FAILED_TO_PASTE_FROM_CLIPBOARD       = '     ';
 E_FAILED_TO_MOVE_TO_SPECIFIED_POSITION = '      : %s';
 E_FAILED_TO_FIND_SPECIFIED_TEXT        = '      ';
 E_FAILED_TO_REPLACE_SPECIFIED_TEXT     = '      ';
 E_FAILED_TO_FORMAT_FONT                = '   ';
 E_FAILED_TO_FORMAT_AS_BOLD             = '     ';
 E_FAILED_TO_FORMAT_AS_UNDERLINE        = '     ';
 E_FAILED_TO_CONVERT_TEXT_TO_TABLE      = '     ';
 E_FAILED_TO_SELECT_ROW_IN_TABLE        = '     ';
 E_FAILED_TO_SELECT_COLUMN_IN_TABLE     = '     ';
 E_FAILED_TO_SELECT_WHOLE_TABLE         = '     ';
 E_FAILED_TO_MERGE_CELLS_IN_TABLE       = '        ';
 E_FAILED_TO_SPLIT_CELLS_IN_TABLE       = '       ';
 E_FAILED_TO_INSERT_ROWS_IN_TABLE       = '     ';
 E_FAILED_TO_INSERT_COLUMN_IN_TABLE     = '     ';
 E_FAILED_TO_INSERT_TEXT_IN_TABLE       = '     ';
 E_FAILED_TO_MOVE_SEL_TO_FIRST_ROW_CELL = '    /      ';
 E_FAILED_TO_MOVE_SEL_TO_LAST_ROW_CELL  = '    /      ';
 E_FAILED_TO_MOVE_SEL_TO_FIRST_COL_CELL =  '    /      ';
 E_FAILED_TO_MOVE_SEL_TO_LAST_COL_CELL  = '    /      ';
 E_FAILED_TO_SELECT_NEXT_CELL_IN_TABLE  = '      ';
 E_FAILED_TO_SELECT_PREV_CELL_IN_TABLE  = '      ';
 E_FAILED_TO_GO_LINE_DOWN               = '     %d  ';
 E_FAILED_TO_GO_LINE_UP                 = '     %d  ';
 E_FAILED_TO_DELETE_WORD                = '    ';
 E_FAILED_TO_GET_WHERE_SELECTION_BEGINS = '        ';
 E_FAILED_TO_GET_WHERE_SELECTION_ENDS   = '        ';
 E_FAILED_TO_GOTO_TO_DOCUMENT_BEGINNING = '     ';
 E_FAILED_TO_GOTO_TO_DOCUMENT_ENDING    = '     ';
 E_FAILED_TO_SELECT_CURRENT             = '         ';
 E_FAILED_TO_SELECT_FROM_TO             = '       %d   %d';
 E_FAILED_TO_ALIGN_PARA                 = '      ';
 E_FAILED_TO_SET_BULLETS                = '    ""';
 E_FAILED_TO_REMOVE_BULLETS             = '    ""';
 E_FAILED_TO_PRINT_DOC                  = '   ';

{- TkaWordBasicWrapper }

constructor TkaWordBasicWrapper.Create;
begin
 inherited Create;

 VarClear(WordBasic);
end;

destructor TkaWordBasicWrapper.Destroy;
begin
 VarClear(WordBasic);

 inherited Destroy;
end;

procedure TkaWordBasicWrapper.Connect;
begin
 try
  CheckIsConnected; // ,     ,      
 except
  try //     WordBasic-
   WordBasic := GetActiveOleObject(S_OLE_OBJECT_CLASS_NAME);
  except
   try //    
    WordBasic := CreateOleObject(S_OLE_OBJECT_CLASS_NAME);
   except
    on E: Exception do
    begin
      raise EWordBasicException.Create(E_FAILED_TO_CONNECT_TO_MS_WORD + CRLF + E.Message);
    end;
   end;
  end;

  //   ,   Word-
  WordVersion := GetWordVersion;
 end;
end;

procedure TkaWordBasicWrapper.Disconnect(DisconnectMode: TkaWordDisconnectMode = dmPrompt);
begin
 try
  if (DisconnectMode >= 0) then WordBasic.FileExit(DisconnectMode);
  VarClear(WordBasic);
 except
  on E: Exception do
  begin
    raise CreateException(E_FAILED_TO_DISCONNECT_FROM_MS_WORD, E);
  end;
 end;
end;

function TkaWordBasicWrapper.GetWordVersionStr: string;
begin
 try
  Result := WordBasic.AppInfo(2); //   Word-   
 except
  raise CreateException(E_FAILED_TO_GET_MS_WORD_VERSION);
 end;
end;

function  TkaWordBasicWrapper.GetWordVersion: TkaWordVersion;
var tmp: string;
begin
 tmp := GetWordVersionStr;
 tmp := Copy(tmp, 1, Pos('.', tmp) - 1); //     

 if (StrToInt(tmp) < 8) then Result := wvOld
  else Result := wvNew;
end;

procedure TkaWordBasicWrapper.CheckIsConnected;
var Flag: boolean;
begin
 Flag := VarIsEmpty(WordBasic);

 if (not Flag) then //    OLE   ,       
  try
   GetWordVersionStr; // ,    Word-,   ,   
  except
   //    ,    ,     Word,       
   VarClear(WordBasic); //    OLE 
   Flag := true;
  end;

 if (Flag) then raise CreateException(E_NO_CONNECTION_ESTABLISHED);
end;

procedure TkaWordBasicWrapper.CreateNewDocument(const TemplateName: string = '');
begin
 try //   
 if (TemplateName <> '') then WordBasic.FileNew(TemplateName) //   
  else WordBasic.FileNew;                                     //     
 except
  on E: Exception do
  begin
   raise CreateExceptionFmt(E_FAILED_TO_CREATE_NEW_DOCUMENT, [TemplateName], E);
  end;
 end;
end;

procedure TkaWordBasicWrapper.CloseDocument(Mode: TkaWordCloseDocumentMode = dmQuit);
begin
 try
  WordBasic.DocClose(Mode)
 except
  on E: Exception do
  begin
   raise CreateException(E_FAILED_TO_CLOSE_DOCUMENT, E);
  end;
 end;
end;

procedure TkaWordBasicWrapper.Show(IsStartOfDoc: boolean = true);
begin
 try
  if (IsStartOfDoc) then StartOfDoc;

  WordBasic.AppShow;
 except
  on E: Exception do
  begin
   raise CreateException(E_FAILED_TO_SHOW_MS_WORD, E);
  end;
 end;
end;

procedure TkaWordBasicWrapper.Hide;
begin
 try
  WordBasic.AppHide;
 except
  on E: Exception do
  begin
   raise CreateException(E_FAILED_TO_HIDE_MS_WORD, E);
  end;
 end;
end;

procedure TkaWordBasicWrapper.Undo;
begin
 try
  WordBasic.EditUndo;
 except
  on E: Exception do
  begin
   raise CreateException(E_FAILED_TO_UNDO_LAST_ACTION, E);
  end;
 end;
end;

procedure TkaWordBasicWrapper.Redo;
begin
 try
  WordBasic.EditRedo;
 except
  on E: Exception do
  begin
   raise CreateException(E_FAILED_TO_REDO_LAST_ACTION, E);
  end;
 end;
end;

procedure TkaWordBasicWrapper.Select(psFrom, psTo: integer);
begin
 try
  WordBasic.SetSelRange(psFrom, psTo); //   <>   <>
 except
  on E: Exception do
  begin
    raise CreateExceptionFmt(E_FAILED_TO_SELECT_FROM_TO, [psFrom, psTo], E);
  end;
 end;
end;

procedure TkaWordBasicWrapper.SelectCurrent;
begin
 try
  Select(SelStart, SelEnd);
 except
  on E: Exception do
  begin
   raise CreateException(E_FAILED_TO_SELECT_CURRENT, E);
   end;
 end;
end;

procedure TkaWordBasicWrapper.FormatFont(FontSize, IsUnderline, IsBold, IsItalic: integer; FontName: string; ColorCode: integer = 0);
begin
 try
  WordBasic.FormatFont(Points := FontSize, Underline := IsUnderline, Color := ColorCode, Font := FontName, Bold := IsBold, Italic := IsItalic);
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_FORMAT_FONT, E);
 end;
end;

procedure TkaWordBasicWrapper.FormatAsBold(IsBold: boolean = true);
begin
 try
  WordBasic.FormatFont(Bold := IsBold);
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_FORMAT_AS_BOLD, E);
 end;
end;

procedure TkaWordBasicWrapper.FormatAsUnderline(IsUnderline: boolean = true);
begin
 try
  WordBasic.FormatFont(Underline := IsUnderline);
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_FORMAT_AS_UNDERLINE, E);
 end;
end;

procedure TkaWordBasicWrapper.InsertText(const Text: string);
begin
 try
  SelStart := WordBasic.GetSelStartPos; //   
  WordBasic.Insert(Text);               //  
  SelEnd := WordBasic.GetSelEndPos;     //     
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_INSERT_TEXT, E);
 end;
end;

procedure TkaWordBasicWrapper.CopySelection;
begin
 try
  WordBasic.EditCopy;
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_COPY_SELECTION, E);
 end;
end;

procedure TkaWordBasicWrapper.CutSelection;
begin
 try
  WordBasic.EditCut;
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_CUT_SELECTION, E);
 end;
end;

procedure TkaWordBasicWrapper.ClearSelection;
begin
 try
  WordBasic.EditClear;
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_CLEAR_SELECTION, E);
 end;
end;

procedure TkaWordBasicWrapper.PasteFromClipboard;
begin
 try
  WordBasic.EditPaste;
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_PASTE_FROM_CLIPBOARD, E);
 end;
end;

procedure TkaWordBasicWrapper.MoveTo(const Dest: string; IsClear: boolean = true);
begin
 try
  FindText(Dest);
  if (IsClear) then ClearSelection;
 except
  on E: Exception do
   raise CreateExceptionFmt(E_FAILED_TO_MOVE_TO_SPECIFIED_POSITION, [Dest], E);
 end;
end;

procedure TkaWordBasicWrapper.FindText(const TextToFind: string);
begin
 try
  WordBasic.EditFind(Find := TextToFind, Replace := '', Wrap := iTrue);
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_FIND_SPECIFIED_TEXT, E);
 end;
end;

procedure TkaWordBasicWrapper.ReplaceText(const TextToFind, ReplaceWith: string; IsReplaceAll: boolean = true);
begin
 try
  WordBasic.EditReplace(Find := TextToFind, Replace := ReplaceWith, ReplaceOne := not IsReplaceAll, ReplaceAll := IsReplaceAll, Wrap := iTrue);
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_REPLACE_SPECIFIED_TEXT, E);
 end;
end;

procedure TkaWordBasicWrapper.ReplaceText(const TextToFind: string; ReplaceWith: integer; IsReplaceAll: boolean = true);
begin
 ReplaceText(TextToFind, IntToStr(ReplaceWith), IsReplaceAll);
end;

procedure TkaWordBasicWrapper.TextToTable(TableBegin, TableEnd, TableColCount: integer);
begin
 try
  Select(TableBegin, TableEnd); //       
  WordBasic.TextToTable(ConvertFrom := 2, NumColumns := TableColCount); //   ,  - ';'
  WordBasic.TableColumnWidth(AutoFit := iTrue); //    
  WordBasic.BorderInSide;                   // 
  WordBasic.BorderOutSide;                  // 
  EndOfDoc;                                 //    ,    
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_CONVERT_TEXT_TO_TABLE, E);
 end;
end;

procedure TkaWordBasicWrapper.TableSelectCell;
begin
 try
  if (GetWordVersion = wvNew) then  WordBasic.WW7_EditGoto(Destination := '\Cell')
   else WordBasic.EditGoTo(Destination := '\Cell');
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_SELECT_ROW_IN_TABLE, E);
 end;
end;

procedure TkaWordBasicWrapper.TableSelectRow;
begin
 try
  WordBasic.TableSelectRow;
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_SELECT_ROW_IN_TABLE, E);
 end;
end;

procedure TkaWordBasicWrapper.TableSelectColumn;
begin
 try
  WordBasic.TableSelectColumn;
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_SELECT_COLUMN_IN_TABLE, E);
 end;
end;

procedure TkaWordBasicWrapper.TableSelectTable;
begin
 try
  WordBasic.TableSelectTable;
 except
  on E: Exception do
  begin
   raise CreateException(E_FAILED_TO_SELECT_WHOLE_TABLE, E);
  end;
 end;
end;

procedure TkaWordBasicWrapper.TableMergeCells;
begin
 try
  WordBasic.TableMergeCells;
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_MERGE_CELLS_IN_TABLE, E);
 end;
end;

procedure TkaWordBasicWrapper.TableSplitCells(Cols: integer = 2);
begin
 try
  WordBasic.TableSplitCells(NumColumns := Cols);
 except
  on E: Exception do
  begin
    raise CreateException(E_FAILED_TO_SPLIT_CELLS_IN_TABLE, E);
  end;
 end;
end;

procedure TkaWordBasicWrapper.TableInsertRow(Rows: integer; IsGotoFirstCell: boolean = true);
begin
 if (Rows > 0) then
  try
   WordBasic.TableInsertRow(NumRows := Rows);
   if (IsGotoFirstCell) then StartOfRow;
  except
   on E: Exception do
    raise CreateException(E_FAILED_TO_INSERT_ROWS_IN_TABLE, E);
  end;
end;

procedure TkaWordBasicWrapper.TableInsertColumn(IsGotoFirstCell: boolean = true);
begin
 try
  WordBasic.TableInsertColumn;
   if (IsGotoFirstCell) then StartOfColumn;
  except
   on E: Exception do
    raise CreateException(E_FAILED_TO_INSERT_COLUMN_IN_TABLE, E);
  end;
end;

procedure TkaWordBasicWrapper.TableInsertText(const Text: string; IsNextCell: boolean = true);
begin
 try
  InsertText(Text);
  if (IsNextCell) then NextCell;
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_INSERT_TEXT_IN_TABLE, E);
 end;
end;

procedure TkaWordBasicWrapper.TableInsertText(const StringArray: array of string; IsNextCell: boolean = true);
var i, ixLast: integer;
begin
 ixLast := High(StringArray);

 if (ixLast >= 0) then
  begin
   for i := Low(StringArray) to ixLast - 1 do
    TableInsertText(StringArray[i], true);

   TableInsertText(StringArray[ixLast], IsNextCell);
  end;
end;

function TkaWordBasicWrapper.StartOfRow(IsSelect: boolean = false): integer;
begin
 try
  Result := WordBasic.StartOfRow(IsSelect);
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_MOVE_SEL_TO_FIRST_ROW_CELL, E);
 end;
end;

function TkaWordBasicWrapper.EndOfRow(IsSelect: boolean = false): integer;
begin
 try
  Result := WordBasic.EndOfRow(IsSelect);
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_MOVE_SEL_TO_LAST_ROW_CELL, E);
 end;
end;

function TkaWordBasicWrapper.StartOfColumn(IsSelect: boolean = false): integer;
begin
 try
  Result := WordBasic.StartOfColumn(IsSelect);
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_MOVE_SEL_TO_FIRST_COL_CELL, E);
 end;
end;

function TkaWordBasicWrapper.EndOfColumn(IsSelect: boolean = false): integer;
begin
 try
  Result := WordBasic.EndOfColumn(IsSelect);
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_MOVE_SEL_TO_LAST_COL_CELL, E);
 end;
end;

procedure TkaWordBasicWrapper.NextCell;
begin
 try
  WordBasic.NextCell;
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_SELECT_NEXT_CELL_IN_TABLE, E);
 end;
end;

procedure TkaWordBasicWrapper.PrevCell;
begin
 try
  WordBasic.PrevCell;
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_SELECT_PREV_CELL_IN_TABLE, E);
 end;
end;

function TkaWordBasicWrapper.LineDown(Lines: integer = 1; IsSelect: boolean = false): integer;
begin
 try
  Result := WordBasic.LineDown(Count := Lines, Select := IsSelect);
 except
  on E: Exception do
   raise CreateExceptionFmt(E_FAILED_TO_GO_LINE_DOWN, [Lines], E);
 end;
end;

function TkaWordBasicWrapper.LineUp(Lines: integer = 1; IsSelect: boolean = false): integer;
begin
 try
  Result := WordBasic.LineUp(Count := Lines, Select := IsSelect);
 except
  on E: Exception do
   raise CreateExceptionFmt(E_FAILED_TO_GO_LINE_UP, [Lines], E);
 end;
end;

procedure TkaWordBasicWrapper.AlignPara(Alignment: integer);
begin
 try
  case Alignment of
   0: WordBasic.LeftPara;
   1: WordBasic.RightPara;
   2: WordBasic.CenterPara;
   3: WordBasic.JustifyPara;
  end;
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_ALIGN_PARA, E);
 end;
end;

procedure TkaWordBasicWrapper.RemoveBullets;
begin
 try
  WordBasic.FormatBulletDefault(iFalse);
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_REMOVE_BULLETS, E);
 end;
end;

procedure TkaWordBasicWrapper.SetBullets;
begin
 try
  WordBasic.FormatBulletDefault(iTrue);
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_SET_BULLETS, E);
 end;
end;

procedure TkaWordBasicWrapper.DelWord;
begin
 try
  WordBasic.DeleteWord;
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_DELETE_WORD, E);
 end;
end;

procedure TkaWordBasicWrapper.UpdateSelStart;
begin
 try
  SelStart := WordBasic.GetSelStartPos;
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_GET_WHERE_SELECTION_BEGINS, E);
 end;
end;

procedure TkaWordBasicWrapper.UpdateSelEnd;
begin
 try
  SelEnd := WordBasic.GetSelEndPos;
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_GET_WHERE_SELECTION_ENDS, E);
 end;
end;

procedure TkaWordBasicWrapper.StartOfDoc;
begin
 try
  WordBasic.StartOfDocument;
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_GOTO_TO_DOCUMENT_BEGINNING, E);
 end;
end;

procedure TkaWordBasicWrapper.EndOfDoc;
begin
 try
  WordBasic.EndOfDocument;
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_GOTO_TO_DOCUMENT_ENDING, E);
 end;
end;

procedure TkaWordBasicWrapper.PrintDocument;
begin
 try
  WordBasic.FilePrintDefault;
 except
  on E: Exception do
   raise CreateException(E_FAILED_TO_PRINT_DOC, E);
 end;
end;

function TkaWordBasicWrapper.CreateException(const Msg: string): Exception;
begin
 Result := EWordBasicException.Create(Msg + CRLF + S_POSSIBLY_NO_MS_WORD);
end;

function TkaWordBasicWrapper.CreateException(const Msg: string; E: Exception): Exception;
begin
 Result := CreateException(Msg + CRLF + E.Message);
end;

function TkaWordBasicWrapper.CreateException(E: Exception; const Msg: string): Exception;
begin
 Result := CreateException(E.Message + CRLF + Msg);
end;

function TkaWordBasicWrapper.CreateExceptionFmt(const Msg: string; const Args: array of const): Exception;
begin
 Result := EWordBasicException.CreateFmt(Msg + CRLF + S_POSSIBLY_NO_MS_WORD, Args);
end;

function TkaWordBasicWrapper.CreateExceptionFmt(const Msg: string; const Args: array of const; E: Exception): Exception;
begin
 Result := CreateExceptionFmt(Msg + CRLF + E.Message, Args);
end;

function TkaWordBasicWrapper.CreateExceptionFmt(E: Exception; const Msg: string; const Args: array of const): Exception;
begin
 Result := CreateExceptionFmt(E.Message + CRLF + Msg, Args);
end;

end.
